/*
    Copyright (C) 2012 Aleix Pol <aleixpol@kde.org>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef FILTEREDPINNEDCONTACTSPROXYMODEL_H
#define FILTEREDPINNEDCONTACTSPROXYMODEL_H

#include <QSortFilterProxyModel>

class FilteredPinnedContactsProxyModel : public QSortFilterProxyModel
{
    Q_OBJECT
    Q_PROPERTY(QAbstractItemModel* sourceModel READ sourceModel WRITE setSourceModel)
    public:
        FilteredPinnedContactsProxyModel(QObject* parent = 0);

        virtual bool filterAcceptsRow(int source_row, const QModelIndex& source_parent) const;
};

#endif // FILTEREDPINNEDCONTACTSPROXYMODEL_H
